"""config file for project"""

import os


def raise_missing(msg: str):
    """Raises Exception if environment variable is missing

    Parameters
    ----------
    msg : str
        Message saying whats missing

    Raises
    ------
    KeyError
        Exception with msg
    """
    raise KeyError(msg)


SEQ_SERVER = os.getenv("SEQ_SERVER") or raise_missing(
    "Missing SEQ_SERVER environment variable"
)
SEQ_APIKEY = os.getenv("SEQ_APIKEY") or raise_missing(
    "Missing SEQ_APIKEY environment variable"
)
ENVIRONMENT = os.getenv("Environment", "NOT-SET")
LOGGER_NAME = "seqpylogger"



"""
Module: containing helper functions
"""
import threading
import functools


def set_interval(interval):
    """Decorator function to add SetInterval

    Parameters
    ----------
    interval : int
        Seconds when every repeat should happen
    """

    def decorator_threaded_timer(func):
        @functools.wraps(func)
        def wrapper_threaded_timer(*args, **kwargs):
            SetInterval(func, interval, args=args, kwargs=kwargs)

        return wrapper_threaded_timer

    return decorator_threaded_timer


def url_add_trailing_slash(uri: str) -> str:
    """Add trailing slash to url if this is valid

    Parameters
    ----------
    uri : str
        Web address to update

    Returns
    -------
    str
        uri or updated uri
    """

    if len(uri) == 0 or uri[-1] == "/":
        return uri

    split_uri = str.split(uri, "/")

    if len(split_uri) == 0:
        return f"{uri}/"
    if len(split_uri[-1]) > 1 and "?" not in split_uri[-1]:
        return f"{uri}/"
    return uri


class SetInterval:
    """SetInterval class inspired bij js setInterval(function, interval)
    Also has args, kwargs functionality
    """

    def __init__(self, function, seconds, *args, **kwargs):
        self.interval_seconds = seconds
        self.repeating_function = function
        self._args = args
        self._kwargs = kwargs
        self._set_next_time()

    def _interval_handler(self, args, kwargs):
        self.repeating_function(*args, **kwargs)
        self._set_next_time()

    def _set_next_time(self):
        thread = threading.Timer(
            interval=self.interval_seconds,
            function=self._interval_handler,
            args=self._args,
            kwargs=self._kwargs,
        )
        thread.daemon = True
        thread.start()



"""
This module returns a Handler

inside the module the follwing happens
Handler is a queueHandler and can add LogRecords to a queue
Next a queueListener(handler)
Gets items from this queue and passes them to a BufferHandler (with a 10 item capacity)
After capacity the BufferHandler flushes the log messages to a custom http sender, for seq
"""

from .seqpylogger import SeqPyLogger



"""
A QueueListener that does nothing to the logrecord
and passes it to its handler
"""

import logging
from queue import Queue
from logging.handlers import QueueListener


class SeqPyLoggerQueueListener(QueueListener):
    """
    A QueueListener that does nothing to the logrecord
    and passes it to its handler
    """

    def __init__(self, queue: Queue, handler: logging.Handler):
        super().__init__(queue, handler)

    def prepare(self, record: logging.LogRecord) -> logging.LogRecord:
        """Pass trough for LogRecord

        Parameters
        ----------
        record : LogRecord

        Returns
        -------
        LogRecord
        """
        return record
